#!/usr/bin/env node
"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));

// src/cli.ts
var import_builtin_modules = __toESM(require("builtin-modules"));
var import_bundle_require = require("bundle-require");
var import_cac = __toESM(require("cac"));
var import_chokidar = require("chokidar");
var import_dedent = __toESM(require("dedent"));
var import_dequal = require("dequal");
var import_fast_glob = __toESM(require("fast-glob"));
var import_fs3 = __toESM(require("fs"));
var import_kleur = __toESM(require("kleur"));
var import_meriyah = require("meriyah");
var import_path2 = __toESM(require("path"));
var import_resolve = __toESM(require("resolve"));
var import_sucrase = require("sucrase");

// src/config.ts
var import_sync = __toESM(require("escalade/sync"));
var import_fs = __toESM(require("fs"));
function readConfig(cwd) {
  let config;
  let configPath;
  const watchPaths = [];
  (0, import_sync.default)(cwd, (dir, names) => {
    watchPaths.push(dir + "/package.json");
    if (names.includes("package.json")) {
      let pkg;
      try {
        pkg = JSON.parse(import_fs.default.readFileSync(dir + "/package.json", "utf8"));
      } catch {
      }
      if (pkg?.codegentool && typeof pkg.codegentool === "object") {
        config = pkg.codegentool;
        configPath = dir + "/package.json";
        return dir;
      }
    }
    if (names.includes(".git")) {
      return dir;
    }
  });
  config = {
    generators: ["generators/**/*.{ts,mts,js,mjs}", "!**/node_modules/**"],
    format: "cjs",
    ...config
  };
  return {
    config,
    configPath,
    watchPaths
  };
}

// src/util.ts
var import_sync2 = __toESM(require("escalade/sync"));
var import_fs2 = require("fs");
var import_path = __toESM(require("path"));
function getOutputFile(file, format) {
  const id = Math.random().toString(36).substring(2, 15);
  const cacheDir = import_path.default.join(nearestNodeModules(file), ".cache/codegentool");
  const outputFile = import_path.default.join(
    cacheDir,
    file.replace(
      /\.([mc]?[tj]s|[tj]sx)$/,
      `.bundled_${id}.${format === "esm" ? "mjs" : "cjs"}`
    )
  );
  (0, import_fs2.mkdirSync)(import_path.default.dirname(outputFile), { recursive: true });
  return outputFile;
}
function nearestNodeModules(file) {
  const dir = (0, import_sync2.default)(import_path.default.dirname(file), (dir2, names) => {
    if (names.includes("node_modules")) {
      return dir2;
    }
  });
  if (dir) {
    return import_path.default.join(dir, "node_modules");
  }
  return "node_modules";
}

// src/cli.ts
var import_serialize_javascript = __toESM(require("serialize-javascript"));
var program = (0, import_cac.default)("codegentool");
program.command("").option("--watch, -w", "Enable file watching and automatic reruns").action((options) => start(process.cwd(), options));
program.parse();
async function start(cwd, options) {
  const { config, configPath, watchPaths } = readConfig(cwd);
  if (configPath) {
    process.chdir(import_path2.default.dirname(configPath));
  }
  const watcher = (0, import_chokidar.watch)([...config.generators, ...watchPaths]);
  const generators = /* @__PURE__ */ new Map();
  const regenerateQueue = /* @__PURE__ */ new Set();
  const regenerate = debounce(() => {
    for (const name of regenerateQueue) {
      const promise = generators.get(name) ?? Promise.resolve();
      promise.then((close) => close?.()).then(() => {
        generators.set(name, generate(name, config));
      });
    }
  }, 100);
  let initialized = false;
  watcher.on("all", (event, name) => {
    if (name.endsWith("package.json")) {
      if (!initialized)
        return;
      const configResult = readConfig(process.cwd());
      if (!(0, import_dequal.dequal)(config, configResult.config) || !(0, import_dequal.dequal)(watchPaths, configResult.watchPaths)) {
        watcher.close();
        generators.forEach(async (close) => (await close)());
        start(cwd, options);
      }
    } else if (event == "add" || event == "change") {
      regenerateQueue.add(name);
      regenerate();
    }
  });
  await new Promise((resolve2) => {
    watcher.once("ready", () => {
      initialized = true;
      resolve2();
    });
  });
  if (options.watch) {
    console.log(`\u2318 Watching for changes...`);
  } else {
    watcher.close();
    generators.forEach(async (close) => (await close)());
  }
}
async function generate(generatorPath, config) {
  const watcher = (0, import_chokidar.watch)([], {
    ignoreInitial: true
  });
  let close = Promise.resolve(() => {
    watcher.close();
  });
  watcher.on(
    "all",
    debounce(() => {
      watcher.close();
      close = generate(generatorPath, config);
    }, 500)
  );
  const resolveModule = (id, importer) => import_resolve.default.sync(id, {
    extensions: [".ts", ".js", ".mts", ".cts", ".mjs", ".cjs"],
    basedir: import_path2.default.dirname(importer)
  });
  const resolvePlugin = {
    name: "resolve-module",
    setup(build) {
      build.onResolve({ filter: /.*/ }, (args) => {
        if (import_builtin_modules.default.includes(args.path)) {
          return;
        }
        try {
          return {
            path: resolveModule(args.path, args.importer)
          };
        } catch {
        }
      });
    }
  };
  const requireOptions = {
    tsconfig: config.tsconfig,
    format: config.format,
    esbuildOptions: {
      sourcemap: "inline",
      logLevel: "silent",
      plugins: [resolvePlugin]
    }
  };
  try {
    console.log(
      `\u25B6 Running %s`,
      import_kleur.default.cyan(import_path2.default.relative(process.cwd(), generatorPath))
    );
    let outputPath;
    const {
      mod: { default: generator },
      dependencies
    } = await (0, import_bundle_require.bundleRequire)({
      ...requireOptions,
      filepath: generatorPath,
      getOutputFile: (file, format) => {
        return outputPath = getOutputFile(file, format);
      }
    });
    if (typeof generator !== "function") {
      throw new Error(`Generator must default export a function`);
    }
    watcher.add(dependencies.filter((file) => file !== generatorPath));
    let filesGenerated = 0;
    const seenErrors = /* @__PURE__ */ new Set();
    const watchPaths = /* @__PURE__ */ new Set();
    const read = (file, options) => {
      file = import_path2.default.resolve(file);
      watchPaths.add(file);
      return import_fs3.default.readFileSync(file, options);
    };
    const api = {
      scan: (source, options) => {
        const cwd = import_path2.default.resolve(options?.cwd || ".");
        if (Array.isArray(source)) {
          source.forEach((s) => watchPaths.add(import_path2.default.resolve(cwd, s)));
        } else {
          watchPaths.add(import_path2.default.resolve(cwd, source));
        }
        return import_fast_glob.default.sync(source, options);
      },
      read,
      write: (file, data) => {
        if (typeof data === "string") {
          let current;
          try {
            current = import_fs3.default.readFileSync(file, "utf8");
          } catch {
          }
          if (data !== current) {
            import_fs3.default.mkdirSync(import_path2.default.dirname(file), { recursive: true });
            import_fs3.default.writeFileSync(file, data);
            filesGenerated++;
            console.log(
              `\u2714\uFE0F Generated %s`,
              import_kleur.default.green(import_path2.default.relative(process.cwd(), file))
            );
          }
        } else {
          let current;
          try {
            current = import_fs3.default.readFileSync(file);
          } catch {
          }
          if (!current?.equals(data)) {
            import_fs3.default.mkdirSync(import_path2.default.dirname(file), { recursive: true });
            import_fs3.default.writeFileSync(file, data);
            filesGenerated++;
            console.log(
              `\u2714\uFE0F Generated %s`,
              import_kleur.default.green(import_path2.default.relative(process.cwd(), file))
            );
          }
        }
      },
      writeEnv: (file, data) => {
        const encodeValue = (value) => JSON.stringify(String(value)).slice(1, -1);
        if (import_fs3.default.existsSync(file)) {
          let changed = false;
          const existing = import_fs3.default.readFileSync(file, "utf8");
          const existingLines = existing.trimEnd().split("\n");
          const foundKeys = /* @__PURE__ */ new Set();
          const newLines = existingLines.map((line) => {
            if (line[0] === "#" || line.trim() === "") {
              return line;
            }
            let [key, value] = line.split("=");
            key = key.trim();
            value = value.trim();
            foundKeys.add(key);
            if (data[key] !== void 0 && data[key] !== value) {
              changed = true;
              if (data[key] === null) {
                return null;
              }
              return `${key}=${encodeValue(data[key])}`;
            }
            return line;
          });
          for (const key of Object.keys(data)) {
            if (!foundKeys.has(key) && data[key] != null) {
              changed = true;
              newLines.push(`${key}=${encodeValue(data[key])}`);
            }
          }
          if (changed) {
            import_fs3.default.writeFileSync(
              file,
              newLines.filter((line) => line !== null).join("\n") + (existing.endsWith("\n") ? "\n" : "")
            );
            console.log(
              `\u2714\uFE0F Updated env file %s`,
              import_kleur.default.green(import_path2.default.relative(process.cwd(), file))
            );
          }
        } else {
          import_fs3.default.writeFileSync(
            file,
            Object.entries(data).filter(([, v]) => v != null).map(([k, v]) => `${k}=${encodeValue(v)}`).join("\n") + "\n"
          );
          console.log(
            `\u2714\uFE0F Created env file %s`,
            import_kleur.default.green(import_path2.default.relative(process.cwd(), file))
          );
        }
      },
      dedent: import_dedent.default,
      serialize: import_serialize_javascript.default,
      parseModule: (file, options) => {
        file = import_path2.default.resolve(file);
        return parse(read(file, "utf8"), file, options);
      },
      parseModuleText: (code, options, file) => {
        return parse(code, file ?? "<unknown>", options);
      },
      loadModule: async (id, basedir) => {
        const isBareSpec = !id.startsWith("./") && !id.startsWith("../");
        try {
          if (!isBareSpec) {
            id = import_path2.default.resolve(basedir ?? import_path2.default.dirname(generatorPath), id);
          }
          id = resolveModule(id, outputPath);
        } catch (error) {
          console.error(error.message);
          return null;
        }
        try {
          if (isBareSpec) {
            return await import(id);
          } else {
            const { mod, dependencies: dependencies2 } = await (0, import_bundle_require.bundleRequire)({
              ...requireOptions,
              filepath: id,
              getOutputFile
            });
            watcher.add(dependencies2);
            return mod;
          }
        } catch (error) {
          const message = error && (error.stack || error.message) || error;
          if (seenErrors.has(message)) {
            return null;
          }
          seenErrors.add(message);
          console.error(`Error loading module "${id}"`);
          if (error.errors) {
            console.error(error.message);
          } else {
            console.error(error);
          }
          return null;
        }
      }
    };
    await generator(api);
    if (!filesGenerated) {
      console.log(
        `\u2714\uFE0F Nothing updated by %s`,
        import_kleur.default.cyan(import_path2.default.relative(process.cwd(), generatorPath))
      );
    }
    watcher.add([...watchPaths]);
  } catch (error) {
    console.error(`Error in ${import_path2.default.resolve(generatorPath)}`);
    console.error(error);
  }
  return async () => (await close)();
}
function debounce(fn, wait) {
  let timeout;
  return function(...args) {
    clearTimeout(timeout);
    timeout = setTimeout(() => fn.apply(this, args), wait);
  };
}
function parse(code, filePath, options) {
  const type = filePath.match(/\.[^.]+$/)?.[0];
  const isJSX = type === ".jsx" || type === ".tsx";
  const isTS = type === ".ts" || type === ".tsx" || type === ".mts" || type === ".cts";
  try {
    let transforms = options?.transforms || [];
    if (isTS && !transforms.includes("typescript")) {
      transforms = [...transforms, "typescript"];
    }
    let transformOptions = options?.transformOptions || {};
    if (isJSX && !transforms.includes("jsx")) {
      transformOptions = { ...transformOptions, jsxRuntime: "preserve" };
      transforms = [...transforms, "jsx"];
    }
    if (transforms.length) {
      const transformResult = (0, import_sucrase.transform)(code, {
        ...options?.transformOptions,
        transforms,
        filePath
      });
      code = transformResult.code;
    }
    return (0, import_meriyah.parseModule)(code, { next: true, jsx: isJSX, ...options });
  } catch (error) {
    error.message = `Error parsing ${filePath}: ${error.message}`;
    throw error;
  }
}
